@@ -48,21 +48,13 @@ static void rlogin_log(Plug plug, int type, SockAddr addr, int port,
 		       const char *error_msg, int error_code)
 {
     Rlogin rlogin = (Rlogin) plug;
-    char addrbuf[256], *msg;
-
-    sk_getaddr(addr, addrbuf, lenof(addrbuf));
-
-    if (type == 0)
-	msg = dupprintf("Connecting to %s port %d", addrbuf, port);
-    else
-	msg = dupprintf("Failed to connect to %s: %s", addrbuf, error_msg);
-
-    logevent(rlogin->frontend, msg);
-    sfree(msg);
+    backend_socket_log(rlogin->frontend, type, addr, port,
+                       error_msg, error_code,
+                       rlogin->conf, !rlogin->firstbyte);
 }
 
-static int rlogin_closing(Plug plug, const char *error_msg, int error_code,
-			  int calling_back)
+static void rlogin_closing(Plug plug, const char *error_msg, int error_code,
+			   int calling_back)
 {
     Rlogin rlogin = (Rlogin) plug;
 
@@ -84,10 +76,9 @@ static int rlogin_closing(Plug plug, const char *error_msg, int error_code,
 	logevent(rlogin->frontend, error_msg);
 	connection_fatal(rlogin->frontend, "%s", error_msg);
     }				       /* Otherwise, the remote side closed the connection normally. */
-    return 0;
 }
 
-static int rlogin_receive(Plug plug, int urgent, char *data, int len)
+static void rlogin_receive(Plug plug, int urgent, char *data, int len)
 {
     Rlogin rlogin = (Rlogin) plug;
     if (urgent == 2) {
@@ -121,7 +112,6 @@ static int rlogin_receive(Plug plug, int urgent, char *data, int len)
 	if (len > 0)
             c_write(rlogin, data, len);
     }
-    return 1;
 }
 
 static void rlogin_sent(Plug plug, int bufsize)
@@ -161,7 +151,7 @@ static void rlogin_startup(Rlogin rlogin, const char *ruser)
  */
 static const char *rlogin_init(void *frontend_handle, void **backend_handle,
 			       Conf *conf,
-			       char *host, int port, char **realhost,
+			       const char *host, int port, char **realhost,
 			       int nodelay, int keepalive)
 {
     static const struct plug_function_table fn_table = {
@@ -194,16 +184,8 @@ static const char *rlogin_init(void *frontend_handle, void **backend_handle,
     /*
      * Try to find host.
      */
-    {
-	char *buf;
-	buf = dupprintf("Looking up host \"%s\"%s", host,
-			(addressfamily == ADDRTYPE_IPV4 ? " (IPv4)" :
-			 (addressfamily == ADDRTYPE_IPV6 ? " (IPv6)" :
-			  "")));
-	logevent(rlogin->frontend, buf);
-	sfree(buf);
-    }
-    addr = name_lookup(host, port, realhost, conf, addressfamily);
+    addr = name_lookup(host, port, realhost, conf, addressfamily,
+                       rlogin->frontend, "rlogin connection");
     if ((err = sk_addr_error(addr)) != NULL) {
 	sk_addr_free(addr);
 	return err;
@@ -279,7 +261,7 @@ static void rlogin_reconfig(void *handle, Conf *conf)
 /*
  * Called to send data down the rlogin connection.
  */
-static int rlogin_send(void *handle, char *buf, int len)
+static int rlogin_send(void *handle, const char *buf, int len)
 {
     Rlogin rlogin = (Rlogin) handle;
 
@@ -425,6 +407,7 @@ Backend rlogin_backend = {
     rlogin_provide_logctx,
     rlogin_unthrottle,
     rlogin_cfg_info,
+    NULL /* test_for_upstream */,
     "rlogin",
     PROT_RLOGIN,
     513
