@@ -31,9 +31,14 @@ static void serial_parity_handler(union control *ctrl, void *dlg,
     };
     int mask = ctrl->listbox.context.i;
     int i, j;
-    Config *cfg = (Config *)data;
+    Conf *conf = (Conf *)data;
 
     if (event == EVENT_REFRESH) {
+	/* Fetching this once at the start of the function ensures we
+	 * remember what the right value is supposed to be when
+	 * operations below cause reentrant calls to this function. */
+	int oldparity = conf_get_int(conf, CONF_serparity);
+
 	dlg_update_start(ctrl, dlg);
 	dlg_listbox_clear(ctrl, dlg);
 	for (i = 0; i < lenof(parities); i++)  {
@@ -42,19 +47,27 @@ static void serial_parity_handler(union control *ctrl, void *dlg,
 				      parities[i].val);
 	}
 	for (i = j = 0; i < lenof(parities); i++) {
-	    if (cfg->serparity == parities[i].val)
-		dlg_listbox_select(ctrl, dlg, j);
-	    if (mask & (1 << i))
+	    if (mask & (1 << i)) {
+		if (oldparity == parities[i].val) {
+		    dlg_listbox_select(ctrl, dlg, j);
+		    break;
+		}
 		j++;
+	    }
+	}
+	if (i == lenof(parities)) {    /* an unsupported setting was chosen */
+	    dlg_listbox_select(ctrl, dlg, 0);
+	    oldparity = SER_PAR_NONE;
 	}
 	dlg_update_done(ctrl, dlg);
+	conf_set_int(conf, CONF_serparity, oldparity);    /* restore */
     } else if (event == EVENT_SELCHANGE) {
 	int i = dlg_listbox_index(ctrl, dlg);
 	if (i < 0)
 	    i = SER_PAR_NONE;
 	else
 	    i = dlg_listbox_getid(ctrl, dlg, i);
-	cfg->serparity = i;
+	conf_set_int(conf, CONF_serparity, i);
     }
 }
 
@@ -72,9 +85,14 @@ static void serial_flow_handler(union control *ctrl, void *dlg,
     };
     int mask = ctrl->listbox.context.i;
     int i, j;
-    Config *cfg = (Config *)data;
+    Conf *conf = (Conf *)data;
 
     if (event == EVENT_REFRESH) {
+	/* Fetching this once at the start of the function ensures we
+	 * remember what the right value is supposed to be when
+	 * operations below cause reentrant calls to this function. */
+	int oldflow = conf_get_int(conf, CONF_serflow);
+
 	dlg_update_start(ctrl, dlg);
 	dlg_listbox_clear(ctrl, dlg);
 	for (i = 0; i < lenof(flows); i++)  {
@@ -82,19 +100,27 @@ static void serial_flow_handler(union control *ctrl, void *dlg,
 		dlg_listbox_addwithid(ctrl, dlg, flows[i].name, flows[i].val);
 	}
 	for (i = j = 0; i < lenof(flows); i++) {
-	    if (cfg->serflow == flows[i].val)
-		dlg_listbox_select(ctrl, dlg, j);
-	    if (mask & (1 << i))
+	    if (mask & (1 << i)) {
+		if (oldflow == flows[i].val) {
+		    dlg_listbox_select(ctrl, dlg, j);
+		    break;
+		}
 		j++;
+	    }
+	}
+	if (i == lenof(flows)) {       /* an unsupported setting was chosen */
+	    dlg_listbox_select(ctrl, dlg, 0);
+	    oldflow = SER_FLOW_NONE;
 	}
 	dlg_update_done(ctrl, dlg);
+	conf_set_int(conf, CONF_serflow, oldflow);/* restore */
     } else if (event == EVENT_SELCHANGE) {
 	int i = dlg_listbox_index(ctrl, dlg);
 	if (i < 0)
-	    i = SER_PAR_NONE;
+	    i = SER_FLOW_NONE;
 	else
 	    i = dlg_listbox_getid(ctrl, dlg, i);
-	cfg->serflow = i;
+	conf_set_int(conf, CONF_serflow, i);
     }
 }
 
@@ -132,7 +158,7 @@ void ser_setup_config_box(struct controlbox *b, int midsession,
 		if (c->radio.shortcuts) {
 		    c->radio.shortcuts =
 			sresize(c->radio.shortcuts, c->radio.nbuttons, char);
-		    c->radio.shortcuts[c->radio.nbuttons-1] = NO_SHORTCUT;
+		    c->radio.shortcuts[c->radio.nbuttons-1] = 'r';
 		}
 	    }
 	}
@@ -155,23 +181,22 @@ void ser_setup_config_box(struct controlbox *b, int midsession,
 			"Select a serial line");
 	ctrl_editbox(s, "Serial line to connect to", 'l', 40,
 		     HELPCTX(serial_line),
-		     dlg_stdeditbox_handler, I(offsetof(Config,serline)),
-		     I(sizeof(((Config *)0)->serline)));
+		     conf_editbox_handler, I(CONF_serline), I(1));
     }
 
     s = ctrl_getset(b, "Connection/Serial", "sercfg", "Configure the serial line");
     ctrl_editbox(s, "Speed (baud)", 's', 40,
 		 HELPCTX(serial_speed),
-		 dlg_stdeditbox_handler, I(offsetof(Config,serspeed)), I(-1));
+		 conf_editbox_handler, I(CONF_serspeed), I(-1));
     ctrl_editbox(s, "Data bits", 'b', 40,
 		 HELPCTX(serial_databits),
-		 dlg_stdeditbox_handler,I(offsetof(Config,serdatabits)),I(-1));
+		 conf_editbox_handler, I(CONF_serdatabits), I(-1));
     /*
      * Stop bits come in units of one half.
      */
     ctrl_editbox(s, "Stop bits", 't', 40,
 		 HELPCTX(serial_stopbits),
-		 dlg_stdeditbox_handler,I(offsetof(Config,serstopbits)),I(-2));
+		 conf_editbox_handler, I(CONF_serstopbits), I(-2));
     ctrl_droplist(s, "Parity", 'p', 40,
 		  HELPCTX(serial_parity),
 		  serial_parity_handler, I(parity_mask));
