@@ -15,11 +15,11 @@ extern "C" {
 #endif /* __cplusplus */
 
 #define TYPE_COLUMNS (columns_get_type())
-#define COLUMNS(obj) (GTK_CHECK_CAST((obj), TYPE_COLUMNS, Columns))
-#define COLUMNS_CLASS(klass) \
-                (GTK_CHECK_CLASS_CAST((klass), TYPE_COLUMNS, ColumnsClass))
-#define IS_COLUMNS(obj) (GTK_CHECK_TYPE((obj), TYPE_COLUMNS))
-#define IS_COLUMNS_CLASS(klass) (GTK_CHECK_CLASS_TYPE((klass), TYPE_COLUMNS))
+#define COLUMNS(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj), TYPE_COLUMNS, Columns))
+#define COLUMNS_CLASS(klass)                                            \
+    (G_TYPE_CHECK_CLASS_CAST((klass), TYPE_COLUMNS, ColumnsClass))
+#define IS_COLUMNS(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), TYPE_COLUMNS))
+#define IS_COLUMNS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass), TYPE_COLUMNS))
 
 typedef struct Columns_tag Columns;
 typedef struct ColumnsClass_tag ColumnsClass;
@@ -42,18 +42,21 @@ struct ColumnsChild_tag {
     GtkWidget *widget;
     gint colstart, colspan;
     gboolean force_left;	       /* for recalcitrant GtkLabels */
+    ColumnsChild *same_height_as;
     /* Otherwise, this entry represents a change in the column setup. */
     gint ncols;
     gint *percentages;
+    gint x, y, w, h;           /* used during an individual size computation */
 };
 
-GtkType columns_get_type(void);
+GType columns_get_type(void);
 GtkWidget *columns_new(gint spacing);
 void columns_set_cols(Columns *cols, gint ncols, const gint *percentages);
 void columns_add(Columns *cols, GtkWidget *child,
                  gint colstart, gint colspan);
 void columns_taborder_last(Columns *cols, GtkWidget *child);
 void columns_force_left_align(Columns *cols, GtkWidget *child);
+void columns_force_same_height(Columns *cols, GtkWidget *ch1, GtkWidget *ch2);
 
 #ifdef __cplusplus
 }
