@@ -41,6 +41,7 @@
 #define WINHELP_CTX_features_qtitle "features.qtitle:config-features-qtitle"
 #define WINHELP_CTX_features_dbackspace "features.dbackspace:config-features-dbackspace"
 #define WINHELP_CTX_features_charset "features.charset:config-features-charset"
+#define WINHELP_CTX_features_clearscroll "features.clearscroll:config-features-clearscroll"
 #define WINHELP_CTX_features_arabicshaping "features.arabicshaping:config-features-shaping"
 #define WINHELP_CTX_features_bidi "features.bidi:config-features-bidi"
 #define WINHELP_CTX_terminal_autowrap "terminal.autowrap:config-autowrap"
@@ -86,6 +87,7 @@
 #define WINHELP_CTX_proxy_dns "proxy.dns:config-proxy-dns"
 #define WINHELP_CTX_proxy_auth "proxy.auth:config-proxy-auth"
 #define WINHELP_CTX_proxy_command "proxy.command:config-proxy-command"
+#define WINHELP_CTX_proxy_logging "proxy.logging:config-proxy-logging"
 #define WINHELP_CTX_telnet_environ "telnet.environ:config-environ"
 #define WINHELP_CTX_telnet_oldenviron "telnet.oldenviron:config-oldenviron"
 #define WINHELP_CTX_telnet_passive "telnet.passive:config-ptelnet"
@@ -101,6 +103,7 @@
 #define WINHELP_CTX_ssh_compress "ssh.compress:config-ssh-comp"
 #define WINHELP_CTX_ssh_share "ssh.sharing:config-ssh-sharing"
 #define WINHELP_CTX_ssh_kexlist "ssh.kex.order:config-ssh-kex-order"
+#define WINHELP_CTX_ssh_hklist "ssh.hostkey.order:config-ssh-hostkey-order"
 #define WINHELP_CTX_ssh_kex_repeat "ssh.kex.repeat:config-ssh-kex-rekey"
 #define WINHELP_CTX_ssh_kex_manual_hostkeys "ssh.kex.manualhostkeys:config-ssh-kex-manual-hostkeys"
 #define WINHELP_CTX_ssh_auth_bypass "ssh.auth.bypass:config-ssh-noauth"
